--******************************************************************************
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 253 - Synthesizable DDR SDRAM Controller
--
--*******************************************************************************
--
--  File name :       controller.vhd
--
--  Description :     
--                    Main DDR SDRAM controller block. This includes the following
--                    features:
--                    - The controller state machine that controls the 
--                    initialization process upon power up, as well as the 
--                    read, write, and refresh commands. 
--                    - Accepts and decodes the user commands.
--                    - Generates the address and Bank address signals
--                    - Generates control signals for other modules, including
--                    the control signals for the dqs_en block.
--
--  Date - revision : 05/01/2002
--
--  Author :          Lakshmi Gopalakrishnan
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--
-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on
--
entity controller_iobs is
port(
    clk0              : in std_logic;
    ddr_rasb_cntrl   : in std_logic;
    ddr_casb_cntrl   : in std_logic;
    ddr_web_cntrl    : in std_logic;
    ddr_cke_cntrl    : in std_logic;
    ddr_csb_cntrl    : in std_logic;
    ddr_address_cntrl: in std_logic_vector(12 downto 0);
    ddr_ba_cntrl     : in std_logic_vector(1 downto 0);
    rst_dqs_div_int  : in std_logic;
    rst_dqs_div_int1  : in std_logic;
    ddr_rasb         : out std_logic;
    ddr_casb         : out std_logic;
    ddr_web          : out std_logic;
    ddr_ba           : out std_logic_vector(1 downto 0);
    ddr_address      : out std_logic_vector(12 downto 0);
    ddr_cke          : out std_logic;
    ddr_csb          : out std_logic;
    rst_dqs_div      : out std_logic;
    rst_dqs_div2     : out std_logic;
    rst_dqs_div_in   : in std_logic;
    rst_dqs_div_in2  : in std_logic;
    rst_dqs_div_out2 : out std_logic; 
    rst_dqs_div_out  : out std_logic
   
);
end controller_iobs;


architecture arc_controller_iobs of controller_iobs is

attribute xc_props : string;
attribute syn_keep : boolean;


component IBUF_SSTL2_II
port (
       I  : in std_logic;
       O  : out std_logic);
end component;

component OBUF_SSTL2_II
port (
       I  : in std_logic;
       O  : out std_logic);
end component;

component IBUF
port (
       I  : in std_logic;
       O  : out std_logic);
end component;

component OBUF
port (
       I  : in std_logic;
       O  : out std_logic);
end component;


component FD
   port(
      Q                              :  out   STD_LOGIC;
      D                              :  in    STD_LOGIC;
      C                              :  in    STD_LOGIC);
end component;



---- **************************************************
---- iob attributes for instantiated FD components
---- **************************************************





signal clk180     : std_logic;
signal GND        : std_logic;
signal ddr_web_q  : std_logic;
signal ddr_rasb_q : std_logic;
signal ddr_casb_q : std_logic;

attribute xc_props of iob_web:  label is "IOB=TRUE";
attribute xc_props of iob_rasb: label is "IOB=TRUE";
attribute xc_props of iob_casb: label is "IOB=TRUE";
 
attribute syn_keep of clk180 : signal is true; 
begin

clk180 <= not clk0;
GND <= '0';
  

---- ******************************************* ----
----  Includes the instantiation of FD for cntrl ----
----            signals                          ----
---- ******************************************* ----
           
iob_web : FD port map (
                         Q    => ddr_web_q,
                         D    => ddr_web_cntrl,
                         C    => clk180);
                         
iob_rasb : FD port map (
                         Q    => ddr_rasb_q,
                         D    => ddr_rasb_cntrl,
                         C    => clk180);
                         
iob_casb : FD port map (
                         Q    => ddr_casb_q,
                         D    => ddr_casb_cntrl,
                         C    => clk180);                                                  
                                                        


---- ************************************* ----
----  Output buffers for control signals   ----
---- ************************************* ----

r0 : OBUF port map (
                     I => ddr_web_q,
                     O => ddr_web);

r1 : OBUF port map (
                     I => ddr_rasb_q,
                     O => ddr_rasb);

r2 : OBUF port map (
                     I => ddr_casb_q,
                     O => ddr_casb);

r3 : OBUF port map (
                     I => ddr_cke_cntrl,
                     O => ddr_cke);

r4 : OBUF port map (
                     I => ddr_csb_cntrl,
                     O => ddr_csb);

---- ************************************* ----
----  Output buffers for address signals   ----
---- ************************************* ----

r5 : OBUF port map (
                     I => ddr_address_cntrl(0),
                     O => ddr_address(0));

r6 : OBUF port map (
                     I => ddr_address_cntrl(1),
                     O => ddr_address(1));

r7 : OBUF port map (
                     I => ddr_address_cntrl(2),
                     O => ddr_address(2));

r8 : OBUF port map (
                     I => ddr_address_cntrl(3),
                     O => ddr_address(3));

r9 : OBUF port map (
                     I => ddr_address_cntrl(4),
                     O => ddr_address(4));

r10 : OBUF port map (
                     I => ddr_address_cntrl(5),
                     O => ddr_address(5));

r11 : OBUF port map (
                     I => ddr_address_cntrl(6),
                     O => ddr_address(6));

r12 : OBUF port map (
                     I => ddr_address_cntrl(7),
                     O => ddr_address(7));

r13 : OBUF port map (
                     I => ddr_address_cntrl(8),
                     O => ddr_address(8));

r14 : OBUF port map (
                     I => ddr_address_cntrl(9),
                     O => ddr_address(9));

r15 : OBUF port map (
                     I => ddr_address_cntrl(10),
                     O => ddr_address(10));

r16 : OBUF port map (
                     I => ddr_address_cntrl(11),
                     O => ddr_address(11));

r17 : OBUF port map (
                     I => ddr_address_cntrl(12),
                     O => ddr_address(12));



r18 : OBUF port map (
                     I => ddr_ba_cntrl(0),
                     O => ddr_ba(0));

r19 : OBUF port map (
                     I => ddr_ba_cntrl(1),
                     O => ddr_ba(1));



---rst_iob_inbuf  :  IBUF port map                        
rst_iob_inbuf  :  IBUF_SSTL2_II port map
                            ( I  => rst_dqs_div_in,
                              O  => rst_dqs_div);
  

---rst_iob_outbuf  :  OBUF port map                            
rst_iob_outbuf  :  OBUF_SSTL2_II port map
                            ( I  => rst_dqs_div_int,
                              O  => rst_dqs_div_out);

--rst_iob_inbuf2  :  OBUF port map
rst_iob_inbuf2  :  IBUF_SSTL2_II port map
                            ( I  => rst_dqs_div_in2,
                              O  => rst_dqs_div2);  
---rst_iob_outbuf2  :  OBUF port map                            
rst_iob_outbuf2  :  OBUF_SSTL2_II port map
                            ( I  => rst_dqs_div_int1,
                              O  => rst_dqs_div_out2);









                             
end arc_controller_iobs;                
         
 




   
